#!/usr/bin/perl

use strict;
use warnings;
use 5.010;

package Tiny::SMTP::Server;

our $VERSION = '1.01.000';

use FindBin qw($Bin);
use lib "$Bin/lib";

# ---------------
use Data::Dumper (qw(Dumper));
local $Data::Dumper::Purity   = 1;
local $Data::Dumper::Useqq    = 1;
local $Data::Dumper::Sortkeys = sub {
    my ($hash) = @_;
    return [ ( sort { lc $a cmp lc $b } keys %$hash ) ];
};
# ---------------

use Net::SMTP::Server;
use Net::SMTP::Server::Client;
#

# DEFAULTS
my $domain = shift(@ARGV) || 'pc1.fritz.box';    # IP des lokalen SMTP-Servers
my $port   = shift(@ARGV) || 25;                 # P des lokalen SMTP-Servers
#
my $debug_log	= 'NUL'; # auf Windows-Pseudodevice NUL wg. Problem mit Log
my $debug       = 0;	# Debug aus
my $force_login = 1;	# SMTP Auth erzwingen

######
# Falls im Testmodus
if ( $ENV{TEST} ) { 
    print STDERR "##### TESTMODE #####\n";
    #warn Dumper(\%ENV);
    $domain      = $ENV{TINY_DOMAIN}      || $domain;
    $port        = $ENV{TINY_PORT}        || $port;
    $debug_log   = $ENV{TINY_DEBUG_LOG}   || $debug_log;
    $debug       = $ENV{TINY_DEBUG}       || $debug;
    $force_login = $ENV{TINY_FORCE_LOGIN} || $force_login;
}
#
######

my $server = Net::SMTP::Server->new( $domain, $port )
  or die "Server $domain:$port cant be created $@";

while ( my $conn = $server->accept() ) { 
	my $client = Net::SMTP::Server::Client->new($conn)
      or die "Unable to handle client connection: $@\n";

    $client->debug($debug);                # 1 = normale Ausgabe an STDERR
                                           # 2 = zus. Dump des $client
    $client->force_login($force_login);    # Versand nur mit Login an SMTP
    $client->debug_log($debug_log);        # Logausgabe des Servers in Datei

    $client->logindb('login.db');

    $client->process;
		
	print STDERR Dumper($client) if $client->debug >= 2;
}

1;
