#!/usr/bin/perl

use strict;
use warnings;

use 5.010;

use utf8;

use Win32;
use Win32::Process;
use FindBin qw($Bin);

# pid Semaphor lesen/schreiben
sub __sema {
    my ($value) = shift;

    if ( not defined $value ) {    # Semaphor für Prozess-ID lesen
        my $id = undef;
        if ( open my $fh, '<', 'tiny.pid' ) {
            my $line = <$fh>;
            if ( length($line) ) {
                ($id) = $line =~ /^(\d+)/;
                #warn $id;
            }
            close $fh;
        }
        return $id;
    }
    else {
        if ( open my $fh, '>', 'tiny.pid' )
        {    # Semaphor für Prozess-ID schreiben
            flock $fh, 0;
            print $fh $value;
            close $fh;
        }

        return 1;
    }
}

# ----------------

my $parm = shift(@ARGV) || '';
my $pid = undef;

if ( $parm eq 'status' ) {
    # pid aus Semphor lesen
    $pid = __sema();
    print STDERR $pid ? "Process running $pid" : "No process running", "\n";

}
elsif ( $parm eq 'start' ) {
    # pid aus Semphor lesen
    $pid = __sema();

    if ( not $pid ) {
        Win32::Process::Create(
            my $ProcessObj,
            'c:\strawberry\perl\bin\perl.exe',
            "perl $Bin\\Tiny-SMTP-Server.pl",
            0,
            Win32::Process::NORMAL_PRIORITY_CLASS()
              | Win32::Process::DETACHED_PROCESS(),
            $Bin,
        ) or die "Process cant be created with Win32::Process::Create!";

        $pid = $ProcessObj->GetProcessID()
          or die "pid of process cant be read with Win32::Process::Create!";

        # Semaphor des Prozess sichern
        __sema($pid);

        print STDERR "Process $pid started", "\n";
    }
    else {
        print STDERR "A process $pid is already running !", "\n";
    }
}
elsif ( $parm eq 'stop' ) {
    $pid = __sema();

    if ($pid) {
        # Semaphor löschen
        __sema('');

        if ( Win32::Process::KillProcess( $pid, 255 ) )
        {    # kill with exit code 42
            print STDERR "Process $pid stopped", "\n";
        }
        else {
            print STDERR 
              "Process $pid cant be stopped with Win32::Process::KillProcess!", "\n";
        }

    }
}
elsif ( $parm =~ /^$|help|\?/ ) {
    print STDERR <<TEXT;
Startup Perl program for Tiny-SMTP-Server

Syntax: 
    server [start|stop|status]

TEXT
}

